<?php

declare(strict_types=1);

namespace Drupal\decoupled_kit\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Decoupled Kit form.
 */
final class DashboardForm extends FormBase {

  /**
   * Drupal\Core\Extension\ModuleHandlerInterface definition.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\DependencyInjection\ContainerInjectionInterface definition.
   *
   * @var \Drupal\Core\DependencyInjection\ContainerInjectionInterface
   */
  protected $decoupledKit;

  /**
   * Current page path for dashboard.
   *
   * @var string
   */
  protected $currentPath;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->moduleHandler = $container->get('module_handler');
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->decoupledKit = $container->get('decoupled_kit');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'decoupled_kit_dashboard';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    // Current page for dashboard.
    if ($this->getRequest()->query->has('current_path')) {
      $current_path = $this->getRequest()->query->get('current_path');
    }
    else {
      $current_path = $this->config('decoupled_kit.config')->get('current_path');
    }
    $current_path = !empty($current_path) ? $this->decoupledKit->canonicalPath($current_path) : '/';

    $form['current_path'] = [
      '#title' => $this->t('Current path for dashboard links'),
      '#type' => 'textfield',
      '#required' => TRUE,
      '#default_value' => $current_path,
      '#weight' => -2,
    ];
    $form['actions'] = [
      '#type' => 'actions',
      '#weight' => -1,
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Set current page'),
    ];

    $form['table'] = [
      '#title' => $this->t('Table of functions'),
      '#type' => 'table',
      '#rows' => $this->getTableRows($current_path),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $current_path = $this->decoupledKit->canonicalPath($form_state->getValue('current_path'));
    $this->configFactory()
      ->getEditable('decoupled_kit.config')
      ->set('current_path', $current_path)
      ->save();
    $form_state->setRedirect('decoupled_kit.dashboard', [], ['query' => ['current_path' => $current_path]]);
  }

  /**
   * Get table rowse.
   *
   * @param string $current_path
   *   Current path.
   */
  protected function getTableRows($current_path): array {
    $functions = [
      [$this->t('Router'), 'decoupled_kit.router'],
    ];

    $modules = [
      'decoupled_kit_block' => [$this->t('Blocks'), 'decoupled_kit.block'],
      'decoupled_kit_redirect' => [$this->t('Redirect'), 'decoupled_kit.redirect'],
    ];

    foreach ($modules as $module => $data) {
      if ($this->moduleHandler->moduleExists($module)) {
        $functions[] = $data;
      }
    }

    $rows = [];
    foreach ($functions as $function) {
      [$title, $route] = $function;
      $url = Url::fromRoute($route, [], [
        'query' => ['current_path' => $current_path],
        'attributes' => ['target' => '_blank'],
      ]);
      $rows[] = [Link::fromTextAndUrl($title, $url)];
    }
    return $rows;
  }

}
